((root, factory) => {
  if (typeof define === 'function' && define.amd) define([], factory);
  else if (typeof exports === 'object') module.exports = factory();
  else root.pointerAPI = factory;
})(this ? this : window, () => {
  const axios = require('axios');
  const sha256 = require('js-sha256').sha256;
  const Cookie = require('js-cookie');
  let pointerAPI;
  let options = {
    apiURL: '',
    digest: false,
    tokenKey: '',
    credentials: {
      key: '',
      code: '',
      pin: ''
    }
  };

  const createDigest = () => {
    const { digest, credentials } = options;
    if (!digest) return '';
    const { key, code, pin } = credentials;
    return sha256(pin + key + code);
  }

  const getStartURL = (resource, id) => {
    const { credentials, digest } = options;
    const { key, code } = credentials;
    let url = `secret-key/${key}/secret-code/${code}/${resource}`;

    if (id !== undefined && id !== null) url += `/${id}`;
    if (digest) url += `?digest=${createDigest()}`;
    return url;
  }

  const pointerGet = (resource = '', params = {}) => pointerAPI.get(getStartURL(resource), { params });
  const pointerPost = (resource = '', params = {}, headers = {}) => pointerAPI.post(getStartURL(resource), params, { headers });
  const pointerPut = (resource = '', params = {}, headers = {}) => pointerAPI.put(getStartURL(resource), params, { headers });
  const pointerDelete = (resource = '', id) => pointerAPI.delete(getStartURL(resource, id));

  let pointerMethods = {
    get: pointerGet,
    post: pointerPost,
    put: pointerPut,
    delete: pointerDelete
  }
  
  const setOptions = (customOptions) => {
    options = {
      ...options,
      ...customOptions
    };
  };

  const setCustomMethods = (customMethods) => {
    pointerMethods = {
      ...pointerMethods,
      customMethods
    }
  }

  const init = () => {
    const token = localStorage.getItem(options.tokenKey);
    const API = axios.create({ baseURL: options.apiURL });

    API.interceptors.request.use((config) => {
      if (token) config.headers.Authorization = token;
      return config;
    }, error => Promise.reject(error));

    API.interceptors.response.use(
      response => response.data,
      async (error) => {
        const originalRequest = error.config;
        if (originalRequest.url.includes('login')) return error.response;
    
        if (error.response.status === 401
          && !originalRequest._retry) {
          const { tokenKey } = options;
          const { data, success } = await instance.post('refresh-token', {
            token: Cookie.get(tokenKey) || sessionStorage.getItem(tokenKey) || token
          });
          if (success) {
            originalRequest.headers.Authorization = `Bearer ${data._token}`;
            return axios((originalRequest));
          }
        }
    
        return error;
      }
    );
    
    pointerAPI = API;
  }

  return {
    setOptions,
    setCustomMethods,
    getStartURL,
    init,
    pointer: pointerMethods
  }
});
